from __future__ import annotations

from typing import Dict

from alicefluentcheck.chain import chained
from alicefluentcheck.constants import *


class AliceIntentSlot:
    """
        Описание одной позиции интента - слота: тип, позиция

        Документация: https://yandex.ru/dev/dialogs/alice/doc/nlu.html

    Methods
    -------
    type(intent_type: str)
        Указать тип слота
    """

    def __init__(self, name: str):
        self.name = name
        self.slot_type = YA_STRING
        self.slot_tokens = {"start": 0, "end": 0}
        self.slot_value = ""

    @chained
    def type(self, intent_type: str) -> AliceIntentSlot:
        """
        Указать тип слота
        Допустимые значения:

            - YANDEX.NUMBER
            - YANDEX.STRING
            - YANDEX.DATETIME
            - YANDEX.GEO
            - YANDEX.FIO

        :param str intent_type: задает тип слота
        """
        assert intent_type in [
            YA_STRING,
            YA_NUMBER,
            YA_GEO,
            YA_DATETIME,
            YA_FIO,
        ], "Неверный тип интента"
        self.slot_type = intent_type

    @chained
    def type_number(self) -> AliceIntentSlot:
        """
        Указать тип слота - число
        """
        self.slot_type = YA_NUMBER

    @chained
    def type_string(self) -> AliceIntentSlot:
        """
        Указать тип слота - строка
        """
        self.slot_type = YA_STRING

    @chained
    def type_datetime(self) -> AliceIntentSlot:
        """
        Указать тип слота - дата/время
        """
        self.slot_type = YA_DATETIME

    @chained
    def type_geo(self) -> AliceIntentSlot:
        """
        Указать тип слота - геоданные
        """
        self.slot_type = YA_GEO

    @chained
    def type_fio(self) -> AliceIntentSlot:
        """
        Указать тип слота - фио человека
        """
        self.slot_type = YA_FIO

    @chained
    def tokens(self, start=0, end=0) -> AliceIntentSlot:
        """
        Описание позиции найденного интента во фразе пользователя

        :param int start: начало слота во фразе
        :param int end: окончания слота во фразе
        """
        assert type(start) is int, "Параметр НАЧАЛО должен быть числом"
        assert type(end) is int, "Параметр ОКОНЧАНИЕ должен быть числом"

        assert 0 <= start <= end, "Нарушен порядок следования"
        self.slot_tokens["start"] = start
        self.slot_tokens["end"] = end

    @chained
    def value(self, value) -> AliceIntentSlot:
        """
        Указать значение слота

        :param value: значение слота
        """
        self.slot_value = value

    @property
    def val(self) -> Dict:
        """
        Собрать значение слота

        :return: значение одной позиции слота интента
        """
        result = {
            self.name: {
                "type": self.slot_type,
                "tokens": {
                    "start": self.slot_tokens["start"],
                    "end": self.slot_tokens["end"],
                },
                "value": self.slot_value,
            }
        }

        return result


class AliceIntent:
    """
    Описание одно интента: имя интента и слоты

    Документация: https://yandex.ru/dev/dialogs/alice/doc/nlu.html
    """

    def __init__(self, name=""):
        self.name = name
        self.slots = {}

    @chained
    def add_slot(self, slot: AliceIntentSlot) -> AliceIntent:
        """

        Добавить слот к интенту: описание позиции где был найден интент, его значение
        :param AliceIntentSlot slot: один слот для интента
        """
        self.slots.update(slot.val)

    # Специфичные интенты

    @chained
    def confirm(self) -> AliceIntent:
        """
        Интент "Согласие пользователя": да, согласен и тд
        """
        self.name = YA_CONFIRM
        self.slots.clear()

    @chained
    def reject(self) -> AliceIntent:
        """
        Интент "Отказ пользователя": нет, не надоё и тд
        """
        self.name = YA_REJECT
        self.slots.clear()

    @chained
    def discard(self) -> AliceIntent:
        """
        Интент "Отказ пользователя": нет, не надоё и тд
        """
        self.name = YA_REJECT
        self.slots.clear()

    @chained
    def help(self) -> AliceIntent:
        """
        Интент "Пользователь просит помощи": помощь, справка и тд
        """
        self.name = YA_HELP
        self.slots.clear()

    @chained
    def what_can_you_do(self) -> AliceIntent:
        """
        Интент "Что умеет навык": что ты умеешь
        """
        self.name = YA_WHAT_CAN_YOU_DO
        self.slots.clear()

    @chained
    def repeat(self) -> AliceIntent:
        """
        Интент "Повтори что было до этого": повтори, еще раз и тд
        """
        self.name = YA_REPEAT
        self.slots.clear()

    @property
    def val(self) -> Dict:
        """
        Собрать значение интента
        :return: значение одно интента: тип и слоты
        """
        return {self.name: self.slots}



import json
from typing import Dict, Union

from fluentcheck import Check

#########################################################
# ВАЛИДАЦИЯ ОТВЕТОВ
#########################################################


class AliceAnswer:
    def __init__(self, answer: Union[str, Dict]):

        if type(answer) is str:
            temp_answer = json.loads(answer)
        else:
            temp_answer = answer

        self.answer = temp_answer

    @property
    def response(self):
        return self.answer.get("response", {})

    @property
    def text(self):
        return self.response.get("text", {})

    @property
    def tts(self):
        return self.response.get("tts", {})

    @property
    def session_state(self):
        return self.answer.get("session_state", {})

    @property
    def user_state(self):
        return self.answer.get("user_state_update", {})

    @property
    def next_scene(self):
        return self.session_state.get("scene")

    @property
    def is_end_of_session(self):
        return self.response.get("end_session", False)

    def get_state_session(self, state: str):
        return self.session_state.get(state, None)

    def get_state_user(self, state: str):
        return self.user_state.get(state, None)

    def has_button(self, name: str) -> bool:
        for button in self.response.get("buttons", []):
            if button["title"] == name:
                return True
        return False



from __future__ import annotations

from typing import Dict

from alicefluentcheck.chain import chained
from alicefluentcheck.constants import *


class AliceEntity:
    """
    Описание одной сущности: имя сущности, состав и положение

    Документация: https://yandex.ru/dev/dialogs/alice/doc/nlu.html
    """

    def __init__(self, entity_type=""):
        self.type = entity_type
        self.entity_tokens = {"start": 0, "end": 0}
        self.entity_value = ""

    @chained
    def tokens(self, start=0, end=0) -> AliceEntity:
        assert 0 <= start <= end, "Нарушен порядок позиций сущности"
        self.entity_tokens["start"] = start
        self.entity_tokens["end"] = end

    @chained
    def value(self, value) -> AliceEntity:
        self.entity_value = value

    @chained
    def fio(self, last_name="", first_name="", patronymic_name="") -> AliceEntity:
        assert (
            first_name or patronymic_name or last_name
        ), "Должна быть указана часть ФИО"

        self.type = YA_FIO
        value = {}
        if first_name:
            value["first_name"] = first_name
        if patronymic_name:
            value["patronymic_name"] = patronymic_name
        if last_name:
            value["last_name"] = last_name

        self.entity_value = value

    @chained
    def number(self, num: int) -> AliceEntity:
        self.type = YA_NUMBER
        self.entity_value = num

    @chained
    def datetime(
        self,
        year=None,
        year_is_relative=None,
        month=None,
        month_is_relative=None,
        day=None,
        day_is_relative=None,
        hour=None,
        hour_is_relative=None,
        minute=None,
        minute_is_relative=None,
    ) -> AliceEntity:
        self.type = YA_DATETIME
        value = {
            "year": year,
            "year_is_relative": year_is_relative,
            "month": month,
            "month_is_relative": month_is_relative,
            "day": day,
            "day_is_relative": day_is_relative,
            "hour": hour,
            "hour_is_relative": hour_is_relative,
            "minute": minute,
            "minute_is_relative": minute_is_relative,
        }
        value = {k: v for k, v in value.items() if v is not None}
        self.entity_value = value

    @chained
    def tomorrow(self):
        self.datetime(day=1, day_is_relative=True)

    @chained
    def yesterday(self):
        self.datetime(day=-1, day_is_relative=True)

    @property
    def val(self) -> Dict:
        """
        Собрать значение сущности
        :return: значение одной сущности: тип, позиции и значение
        """
        return {
            "type": self.type,
            "value": self.entity_value,
            "tokens": self.entity_tokens,
        }



from alicefluentcheck import AliceEntity, AliceIntent
from alicefluentcheck.chain import chained

#########################################################
# ПОДГОТОВКА ЗАПРОСОВ
#########################################################


class AliceRequest:
    def __init__(
        self,
        has_screen=False,
        has_payments=False,
        has_account_linking=False,
        has_audio_player=False,
    ):
        self._new_session = False
        self._command = ""
        self._original_utterance = ""
        self._nlu_tokens = []
        self._intents = {}
        self._entities = []
        self._state_sessions = {}
        self._state_user = {}
        self._state_application = {}
        self._interfaces = {}
        self._access_token = None
        self._account_linking_complete_event = False
        if has_screen:
            self._interfaces["screen"] = {}
        if has_payments:
            self._interfaces["payments"] = {}
        if has_account_linking:
            self._interfaces["account_linking"] = {}
        if has_audio_player:
            self._interfaces["audio_player"] = {}

    @chained
    def new_session(self):
        self._new_session = True

    @chained
    def command(self, command=""):
        self._command = command.lower()
        self.original_utterance(command)

    @chained
    def original_utterance(self, original_utterance=""):
        self._original_utterance = original_utterance
        self.nlu_token(original_utterance.lower().split(" "))

    @chained
    def access_token(self, access_token="777"):
        self._access_token = access_token

    @chained
    def nlu_token(self, tokens: list):
        self._nlu_tokens = tokens

    @chained
    def add_intent(self, intent: AliceIntent):
        self._intents.update(intent.val)

    @chained
    def add_entity(self, entity: AliceEntity):
        self._entities.append(entity.val)

    @chained
    def add_to_state_session(self, name: str, value):
        self._state_sessions[name] = value

    @chained
    def add_to_state_user(self, name: str, value):
        self._state_user[name] = value

    @chained
    def add_to_state_application(self, name: str, value):
        self._state_application[name] = value

    @chained
    def from_scene(self, scene: str):
        self.add_to_state_session("scene", scene)

    @chained
    def account_linking_complete(self):
        self._account_linking_complete_event = True

    def build(self):
        def meta():
            return {
                "locale": "ru-RU",
                "timezone": "UTC",
                "client_id": "AliceMock",
                "interfaces": self._interfaces,
            }

        def session(new=False):
            temp = {
                "message_id": 3,
                "session_id": "d825cbef-e7d6-4af9-9810-3ff3f358ac16",
                "skill_id": "3308dc06-b901-4f7e-8882-beb1b84c0753",
                "user": {"user_id": "000"},
                "application": {"application_id": "000"},
                "user_id": "000",
                "new": new,
            }
            if self._access_token is not None:
                temp["user"]["access_token"] = self._access_token

            return temp

        if self._account_linking_complete_event:
            req = {
                "meta": meta(),
                "session": session(self._new_session),
                "account_linking_complete_event": {},
                "version": "1.0",
                "state": {
                    "session": self._state_sessions,
                    "user": self._state_user,
                    "applications": self._state_application,
                },
            }

        else:
            req = {
                "meta": meta(),
                "session": session(self._new_session),
                "request": {
                    "command": self._command,
                    "original_utterance": self._original_utterance,
                    "nlu": {
                        "tokens": self._nlu_tokens,
                        "entities": self._entities,
                        "intents": self._intents,
                    },
                    "markup": {"dangerous_context": False},
                    "type": "SimpleUtterance",
                },
                "version": "1.0",
                "state": {
                    "session": self._state_sessions,
                    "user": self._state_user,
                    "applications": self._state_application,
                },
            }
        return req



from alicefluentcheck.AliceAnswer import AliceAnswer
from alicefluentcheck.AliceEntity import AliceEntity
from alicefluentcheck.AliceIntents import AliceIntent, AliceIntentSlot
from alicefluentcheck.AliceRequest import AliceRequest



import os
import re

from setuptools import find_namespace_packages, setup

package_version = os.environ.get("VERSION", "0.0.0.dev0")

if not re.match(r"\d+\.\d+\.\d(\..*)?", package_version):
    package_version = "0.0.0-dev"
with open("README.md", "r", encoding="utf-8") as readme:
    long_description = readme.read()

title = "alicefluentcheck"
author = "a.krapivin"
email = "krapivin_andrey@mail.ru"
url = "https://github.com/KrapivinAndrey/YaAlice_FluentTesting"

requires = ["fluentcheck"]
requires_test = ["pytest", "pytest-cov", "pytest-runner"]
requires_dev = requires_test + ["black", "flake8"]

setup(
    name=title,
    version=package_version,
    description="Подготовка запросов для тестирования навыков Яндекс.Алисы",
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires=">=3.7",
    install_requires=requires,
    setup_requires=requires_dev,
    tests_require=requires_test,
    url=url,
    packages=["alicefluentcheck"],
    include_package_data=True,
    author=author,
    author_email=email,
    maintainer=author,
    maintainer_email=email,
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Operating System :: OS Independent",
    ],
)



YA_STRING: str = "YANDEX.STRING"
"""Тип строка"""

YA_NUMBER: str = "YANDEX.NUMBER"
"""Тип число"""

YA_DATETIME: str = "YANDEX.DATETIME"
"""Тип дата, может быть относительной"""

YA_FIO: str = "YANDEX.FIO"
"""Тип ФИО"""

YA_GEO: str = "YANDEX.GEO"
"""Тип географические данные"""

# Специфичные типы

YA_CONFIRM: str = "YANDEX.CONFIRM"
"""Тип согласие пользователя"""

YA_REJECT: str = "YANDEX.REJECT"
"""Тип отказ пользователя"""

YA_HELP: str = "YANDEX.HELP"
"""Тип пользователь запросил помощь"""

YA_REPEAT: str = "YANDEX.REPEAT"
"""Тип пользователь попросил повторить"""

YA_WHAT_CAN_YOU_DO: str = "YANDEX.WHAT_CAN_YOU_DO"
"""Тип пользователь спросил что может навык
    
    Документация: https://yandex.ru/dev/dialogs/alice/doc/requirements.html#specific__content"""
